package com.talanx.harpoon.process.multiinstance.nonarquillian;

import static org.camunda.bpm.engine.test.assertions.ProcessEngineAssertions.assertThat;
import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.enterprise.inject.spi.BeanManager;
import javax.inject.Inject;

import org.camunda.bpm.engine.cdi.impl.util.BeanManagerLookup;
import org.camunda.bpm.engine.impl.util.LogUtil;
import org.camunda.bpm.engine.runtime.ProcessInstance;
import org.camunda.bpm.engine.test.Deployment;
import org.camunda.bpm.engine.test.ProcessEngineRule;
import org.junit.Before;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;

import com.talanx.harpoon.test.HarpoonJUnitRunner;

/**
 * Test case starting an in-memory database-backed Process Engine.
 */
@RunWith(HarpoonJUnitRunner.class)
public class InMemoryH2Test {

	@Rule
	public ProcessEngineRule rule = new ProcessEngineRule();

	private static final String PROCESS_DEFINITION_KEY = "multiinstance";

	@Inject
	private BeanManager beanManager;

	static {
		LogUtil.readJavaUtilLoggingConfigFromClasspath();
	}

	// @After
	// public void tearDown() {
	// // calculate coverage for all tests
	// ProcessTestCoverage.calculate(processEngine());
	// }

	@Before
	public void setup() {
		BeanManagerLookup.localInstance = this.beanManager;
	}

	/**
	 * Just tests if the process definition is deployable.
	 */
	@Test
	@Deployment(resources = "process.bpmn")
	public void testParsingAndDeployment() {
		// nothing is done here, as we just want to check for exceptions during deployment
	}

	/**
	 * Just tests if the process definition is deployable.
	 */
	@Test
	@Deployment(resources = "process.bpmn")
	public void testExecution() {
		List<String> stringList = new ArrayList<String>();
		for (int i = 0; i < 2; i++) {
			stringList.add("Iteration " + i);
		}
		Map<String, Object> objectMap = new HashMap<String, Object>();
		objectMap.put("list", stringList);
		ProcessInstance pi = this.rule.getRuntimeService().startProcessInstanceByKey(PROCESS_DEFINITION_KEY, objectMap);
		assertEquals(2, ((List<String>) this.rule.getRuntimeService().getVariable(pi.getProcessInstanceId(), "list")).size());
		assertThat(pi).isWaitingAt("loggerTask");
		String jobId = this.rule.getManagementService().createJobQuery().executable().singleResult().getId();
		this.rule.getManagementService().executeJob(jobId);
		String executionId = this.rule.getRuntimeService().createExecutionQuery().list().get(0).getId();
		this.rule.getRuntimeService().signal(executionId);

		this.rule.getRuntimeService().createExecutionQuery();
	}

}
